/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************
 *   Maarten de Boer <maarten.deboer@iua.upf.es>, 1999                     *
 *   Music Technology Group                                                *
 *   Institut de l'Audiovisual, Universitat Pompeu Fabra, Barcelona, Spain *
 *   http://www.iua.upf.es/mtg/                                            *
 ***************************************************************************/

#include "SDIFDisplayAxis.H"
#include "SDIFDisplay.H"
#include <FL/fl_draw.H>
#include <math.h>
#include <stdio.h>

SDIFDisplayAxis::SDIFDisplayAxis(int X,int Y,int W,int H,int T,SDIFDisplay* d)
:Fl_Widget(X,Y,W,H)
{
  mDisplay = d;
  type(T);
}

void valuestr(double tmp,char* pText,double value,int dec)
{
  char f[256];
  if (value == 0)
  {
    snprintf(pText,250,"%.0f",value);
  }
  else
  {
    dec = int(-tmp)-dec;
    if (dec<0) dec=0; 
    if (dec>5) dec=5; 
    snprintf(f,256,"%%.%df",dec);
    snprintf(pText,250,f,value);
  }
}

void SDIFDisplayAxis::draw(void)
{
  fl_push_clip(x(),y(),w(),h());
  
  // draw horitzontal ruler
  fl_font(FL_HELVETICA,8);
  fl_color(color());
  fl_rectf(x(),y(),w(),h());
  
  if (type()==FL_HOR_AXIS)
  {
    fl_color(FL_DARK2);      
    fl_xyline(x(),y()+h()-1,x()+w());

    double offset = mDisplay->HValueOffset();
    double range = w()* mDisplay->DHValue();
    double tmp = log10(range/10.0);
    if (tmp > 0) tmp = (int)(tmp+.1);
    else tmp = (int)(tmp-.9);
    double base = pow(10.0, (int)(tmp)); // ruler scale value
    int ndiv = int(range/base) + 1; // n divisions
    double firstValue = ((int)(offset / base + 0.5)) * base;
    double factor = (double)w() / (range); //
    int divPixels = int(base * factor);
    int i;
    for(i=0; i<ndiv; i++)
    {
      double height;
      double value = firstValue+base*i;
      int xPos = int((value - offset) * factor) + x();
      double dif = value/base/10.0 - (int)(value/base/10.0+0.05);
      if (dif < 0)
        dif = -dif;
      if (dif < 0.05)
      {
        height = 9;
        char pText[250];

        valuestr(tmp,pText,value,1);
        fl_color(FL_BLACK);
        fl_draw(pText,xPos+1,y()+h()-height+3);
      }
      else
      {
        height = 5;
        char pText[250];
        if (divPixels > 30)
        {
          valuestr(tmp,pText,value,0);
          fl_color(FL_BLACK);
          fl_draw(pText,xPos+1,y()+h()-height+3);
        }
      }
      fl_color(FL_DARK2);
      fl_yxline(xPos-1,y()+h()-1,y()+h()-(height-1));
      fl_color(FL_BLACK);
      fl_yxline(xPos,y()+h()-1,y()+h()-(height-1));
      fl_color(FL_WHITE);
      fl_yxline(xPos+1,y()+h()-1,y()+h()-(height-1));
    }
  }
  else
  {
    fl_color(FL_DARK2);      fl_yxline(x()+w()-1,y(),y()+h());
    
    double range = - h()* mDisplay->DVValue();
    double offset = mDisplay->VValueOffset() - range;
    double tmp = log10(fabs(range)/10.0);
    if (tmp > 0) tmp = (int)(tmp+.7);
    else tmp = (int)(tmp-.3);
    double base,firstValue;
    double factor = (double)h() / range; 
    if (range > 0)
    {
      base = pow(10.0, (int)tmp); // ruler scale value
      firstValue = ((int)(offset / base + 0.5)) * base;
    }
    else
    {
      base = -pow(10.0, (int)tmp); // ruler scale value
      firstValue = ((int)(offset / base - 0.5)) * base;
    }
    int ndiv = int(fabs(range/base)) + 3; // n divisions
    int divPixels = int(base* factor);
    int i;
    for(i=0; i<ndiv; i++)
    {
      double width;
      double dif;
      double value = firstValue+base*i;
      int yPos = int((value - offset) * factor) + y();
      double tmp2 = value/base/10.0;
      if (tmp2 > 0)
        dif = tmp2 - (int)(tmp2 + 0.005);
      else
        dif = tmp2 - (int)(tmp2 - 0.005);
      if (dif < 0)
        dif = -dif;
      if (dif < 0.05)
      {
        width = 9;
        char pText[250];

        valuestr(tmp,pText,value,1);
        fl_color(FL_BLACK);
        fl_draw(pText,x()+w()-width+4,yPos,0,0,
          (Fl_Align)(FL_ALIGN_RIGHT|FL_ALIGN_TOP));
      }
      else
      {
        width = 5;
        if (ndiv < 25)
        {
          int DrawText = false;
          if ((divPixels >= 5) && (divPixels < 15))
          {
            if (((int)(dif*100 + 1) % 50) < 5)
              DrawText = true;
          }
          else
          if ((divPixels >= 15) && (divPixels < 20))
          {
            if (((int)(dif*100 + 1) % 20) < 5)
              DrawText = true;
          }
          else
          if (divPixels >= 20)
            DrawText = true;
          if (DrawText)
          {
            char pText[250];

            valuestr(tmp,pText,value,0);
            fl_color(FL_BLACK);
            fl_draw(pText,x()+w()-width,yPos,0,0,
              (Fl_Align)(FL_ALIGN_RIGHT|FL_ALIGN_TOP));
          }
        }
      }
      if (yPos<y()+h()) {
        fl_color(FL_DARK2);
        fl_xyline(x()+w()-3,yPos-1,x()+w()-width);
        fl_color(FL_BLACK);
        fl_xyline(x()+w()-1,yPos,x()+w()-width);
        fl_color(FL_WHITE);
        fl_xyline(x()+w()-2,yPos+1,x()+w()-width);
      }
    }
  }
  
  fl_pop_clip();
}

