/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************
 *   Maarten de Boer <maarten.deboer@iua.upf.es>, 1999                     *
 *   Music Technology Group                                                *
 *   Institut de l'Audiovisual, Universitat Pompeu Fabra, Barcelona, Spain *
 *   http://www.iua.upf.es/mtg/                                            *
 ***************************************************************************/

#include "SDIFDisplayContainer.H"
#include "SDIFDisplayAxis.H"
#include <Fl/Fl_Slider.H>
#include <Fl/Fl_Button.H>
#include <Fl/Fl_Box.H>
#include "ZoomSlider.H"
#include "SDIFDisplayTRC.H"
#include "SDIFDisplaySTF.H"
#include "SDIFDisplayFQ0.H"
#include "SDIFDisplayColors.H"
#include <math.h>
#include <FL/Fl_Valuator.H>

#define TITLEHEIGHT 20
#define SLIDERTHICKNESS 20
#define RULERHEIGHT 14
#define RULERWIDTH 30

SDIFDisplayContainer::SDIFDisplayContainer(
	int X,int Y,int W,int H,char* title,
	SDIFStandardType t,SDIF::Stream* s)
:Fl_Group(X,Y,W,H){
	box(FL_FLAT_BOX);

	hslider = new ButtonedZoomSlider(
		X+RULERWIDTH,Y+H-SLIDERTHICKNESS,
		W-RULERWIDTH-SLIDERTHICKNESS,SLIDERTHICKNESS,
		FL_HORIZONTAL
		);
	vslider = new ButtonedZoomSlider(
		X+W-SLIDERTHICKNESS,Y+TITLEHEIGHT+RULERHEIGHT,
		SLIDERTHICKNESS,H-TITLEHEIGHT-RULERHEIGHT-SLIDERTHICKNESS,
		FL_VERTICAL
		);
	display = 0;

	switch (t) {
	case SDIFStandardTypeSTF:
		display = new SDIFDisplaySTF(
			X+RULERWIDTH,Y+TITLEHEIGHT+RULERHEIGHT,
			W-RULERWIDTH-SLIDERTHICKNESS,H-TITLEHEIGHT-RULERHEIGHT-SLIDERTHICKNESS,s);
		break;
	case SDIFStandardTypeFQ0:
		display = new SDIFDisplayFQ0(
			X+RULERWIDTH,Y+TITLEHEIGHT+RULERHEIGHT,
			W-RULERWIDTH-SLIDERTHICKNESS,H-TITLEHEIGHT-RULERHEIGHT-SLIDERTHICKNESS,s);
		break;
	case SDIFStandardTypeTRC:
		display = new SDIFDisplayTRC(
			X+RULERWIDTH,Y+TITLEHEIGHT+RULERHEIGHT,
			W-RULERWIDTH-SLIDERTHICKNESS,H-TITLEHEIGHT-RULERHEIGHT-SLIDERTHICKNESS,s);
		break;
	}

	vaxis = new SDIFDisplayAxis(X,Y+TITLEHEIGHT+RULERHEIGHT,
		RULERWIDTH,H-TITLEHEIGHT-RULERHEIGHT-SLIDERTHICKNESS,FL_VER_AXIS,display);
	haxis = new SDIFDisplayAxis(X+RULERWIDTH,Y+TITLEHEIGHT,
		W-RULERWIDTH-SLIDERTHICKNESS,RULERHEIGHT,FL_HOR_AXIS,display);

	{
		Fl_Group* g = new Fl_Group(X,Y,W,TITLEHEIGHT);
		g->color(10);
		g->box(FL_THIN_DOWN_BOX);
		Fl_Box* tb = new Fl_Box(X,Y+2,100,TITLEHEIGHT-4,title);
		tb->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
		tb->labelfont(4);
		tb->labelsize(12);
		tb->labelcolor(FL_WHITE);
		Fl_Box* b = new Fl_Box(X+100,Y+4,W-200,TITLEHEIGHT-8);
		g->resizable(b);
	
		if (display->ColorMapping()) {
			colorsslider = new Fl_Slider(X+W-104,Y+3,100,TITLEHEIGHT-7,"Color");
			colorsslider->align(FL_ALIGN_LEFT);
			colorsslider->labelfont(4);
			colorsslider->labelsize(10);			
			colorsslider->labelcolor(FL_WHITE);
			colorsslider->box(FL_THIN_DOWN_BOX);
			colorsslider->type(FL_HORIZONTAL);
			colorsslider->callback((Fl_Callback*)colcb,this);
			colorsslider->value(display->ColorMapping()->value());
		}else{
			colorsslider=0;
		}	

		g->end();
	}

	hslider->callback((Fl_Callback*)hcb,this);
	vslider->callback((Fl_Callback*)vcb,this);

	resizable(display);

	end();
}

void SDIFDisplayContainer::hcb(ButtonedZoomSlider* s,SDIFDisplayContainer* d)
{
	d->display->HOffset(s->offset());
	d->display->HScale(s->scale());
	d->do_callback();
	d->display->redraw();

	if (d->haxis)
		d->haxis->redraw();
}

void SDIFDisplayContainer::vcb(ButtonedZoomSlider* s,SDIFDisplayContainer* d)
{

	d->display->VOffset(s->offset());
	d->display->VScale(s->scale());
	d->do_callback();
	d->display->redraw();
	if (d->vaxis)
		d->vaxis->redraw();
}

void SDIFDisplayContainer::colcb(Fl_Slider* s,SDIFDisplayContainer* d)
{
	d->display->ColorMapping()->value(s->value());
	d->display->redraw();
}

/*********************************************************************/

ButtonedZoomSlider::ButtonedZoomSlider(int X,int Y,int W,int H,int T)
:Fl_Group(X,Y,W,H)
{
	if (T==FL_HORIZONTAL)
	{
			mSlider = new ZoomSlider(X,Y,W-H+2,H,T);
			mZoom = new DragValue(X+W-H+3,Y+1,H-2,H-2);
	}else{
		mSlider = new ZoomSlider(X,Y,W,H-W+2,T);		
		mZoom = new DragValue(X+1,Y+H-W+3,W-2,W-2);
	}

	mZoom->range(0,1);
	mZoom->value(1);
	mZoom->step(0.001);
	mZoom->callback((Fl_Callback*)zoomcb);
	mSlider->callback((Fl_Callback*)slidercb);
	end();
}

void ButtonedZoomSlider::zoomcb(Fl_Valuator* v)
{
	ButtonedZoomSlider* bz = (ButtonedZoomSlider*)v->parent();
	if (bz->mSlider->offset() + v->value()>1.)
	{
		v->value(1.-bz->mSlider->offset());
	}
	bz->mSlider->scale(v->value());
	bz->mSlider->redraw();
	bz->mSlider->do_callback();
}

void ButtonedZoomSlider::slidercb(ZoomSlider* s)
{
	ButtonedZoomSlider* bz = (ButtonedZoomSlider*)s->parent();
	bz->mZoom->value(s->scale());
	bz->do_callback();
}


