/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************
 *   Maarten de Boer <maarten.deboer@iua.upf.es>, 1999                     *
 *   Music Technology Group                                                *
 *   Institut de l'Audiovisual, Universitat Pompeu Fabra, Barcelona, Spain *
 *   http://www.iua.upf.es/mtg/                                            *
 ***************************************************************************/

#include "SDIFDisplayFQ0.H"
#include "SDIFDisplayColors.H"
#include <FL/fl_draw.H>

SDIFDisplayFQ0::SDIFDisplayFQ0(int X,int Y,int W,int H,SDIF::Stream* s)
:SDIFDisplay(X,Y,W,H,s)
{
	mColorMapping = new SDIFDisplayColors(0.5);
}

void SDIFDisplayFQ0::ValuesAtTime(double *freq,double* confidence,double time,
	bool quick)
{
	int i;

	MoveIteratorTo(time);

	for (i=0;i<256;i++) {
		freq[i]=0;
		confidence[i]=0;
	}

	SDIF::Frame* left = *mFrameIterator;
	SDIF::ConcreteMatrix<float>* leftm = 
		dynamic_cast< SDIF::ConcreteMatrix<float>* >(left->GetMatrixAt(0));

	if (quick) {
		for (i=0;i<leftm->Rows() && i<256;i+=8) {
			float f,a=1;
			f = leftm->GetValue(i,0);
			if (leftm->Columns()>1) {
				a = leftm->GetValue(i,1);
			}
			freq[i]=f;
			confidence[i]=a;
		}

		return;
	}

	SDIF::Frame* right = 0;

	{
		SDIF::Stream::FrameIterator next = mFrameIterator;
		next++;
	
		if (next!=mStream->End())
		{
			right	= *(next);
		}
	}
	
	double wr = 0;
	if (right) {
		wr = (time - left->Time())/(right->Time() - left->Time());
	}
	double wl = 1.-wr;
	
	for (i=0;i<leftm->Rows() && i<256;i++) {
		float f,a=1;
		f = leftm->GetValue(i,0);
		if (leftm->Columns()>1) {
			a = leftm->GetValue(i,1);
		}
		freq[i]=f;
		confidence[i]=a*wl;
	}

	if (right)
	{
		SDIF::ConcreteMatrix<float>* rightm =
				dynamic_cast< SDIF::ConcreteMatrix<float>* >(right->GetMatrixAt(0));

		for (i=0;i<rightm->Rows() && i<256;i++) {
			float f,a=1;
			f = rightm->GetValue(i,0);
			if (rightm->Columns()>1) {
				a = rightm->GetValue(i,1);
			}
			if (freq[i] && f!=0.) freq[i]=freq[i]*wl+f*wr;
			else freq[i]=f;
			confidence[i]+=a*wr;
		}
	}
}

void SDIFDisplayFQ0::draw(void)
{
	int i;
	
	fl_color(FL_BLACK);
	fl_rectf(x(),y(),w(),h());

	mFrameIterator = mStream->Begin();

	double t= HValueOffset();
	double dt= DHValue();

	double foffset = VValueOffset();
	double df = DVValue();
	
	int cx=x();

	fl_push_clip(x(),y(),w(),h());

	double freq[256];
	double confidence[256];

	fl_color(FL_WHITE);

	int prevy[256];
	for (i=0;i<256;i++) {
		prevy[i]=0x10000000;
	}
	
	for (cx=x();cx<x()+w();cx++) {
		ValuesAtTime(freq,confidence,t);
		for (i=0;i<256;i++) {
			if (confidence[i]) {
				fl_color(mColorMapping->Get(confidence[i]));
				
				int cy = y()+h()-int((freq[i]-foffset)/df);
				if (prevy[i]!=0x10000000) {
					fl_yxline(cx-1,cy,prevy[i]);
				}else{
					fl_yxline(cx,cy,cy);
				}
				prevy[i]=cy;
			}else{
				prevy[i]=0x10000000;
			}
		}
		t+=dt;
	}
	fl_pop_clip();	
}
