/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************
 *   Maarten de Boer <maarten.deboer@iua.upf.es>, 1999                     *
 *   Music Technology Group                                                *
 *   Institut de l'Audiovisual, Universitat Pompeu Fabra, Barcelona, Spain *
 *   http://www.iua.upf.es/mtg/                                            *
 ***************************************************************************/

#include "SDIFDisplayColors.H"
#include "SDIFDisplaySTF.H"
#include <FL/fl_draw.H>
#include <FL/Fl.H>
#include <math.h>

SDIFDisplaySTF::SDIFDisplaySTF(int X,int Y,int W,int H,SDIF::Stream* s)
:SDIFDisplay(X,Y,W,H,s) {
	mColorMapping = new SDIFDisplayColors(0.5);
}

void SDIFDisplaySTF::ValuesAtTime(double* amp,double time,bool quick)
{
	int i,n;
	static int k=0;
	k++;

	while (mFrameIterator!=mStream->End() &&
		time > (*mFrameIterator)->Time())
	{
		mFrameIterator++;
	}
	if (mFrameIterator==mStream->End()) mFrameIterator--;

	SDIF::Frame* left = *mFrameIterator;
	SDIF::ConcreteMatrix<float>* leftm = 
		dynamic_cast< SDIF::ConcreteMatrix<float>* >(left->GetMatrixAt(1));
	
	n = leftm->Rows();
	for (i=0;i<n;i++) {
		amp[i]=0;
	}

	if (quick) {
		for (i=k&4;i<n;i+=8) {
			float re,im;
			re = leftm->GetValue(i,0);
			im = leftm->GetValue(i,1);
			float a = sqrt(re * re + im * im);
			amp[i]=a;
		}
		return;	
	}

	SDIF::Frame* right = 0;
	
	{
		SDIF::Stream::FrameIterator next = mFrameIterator;
		next++;
	
		if (next!=mStream->End())
		{
			right	= *(next);
		}
	}
	
	double wr = 0;
	if (right) {
		wr = (time - left->Time())/(right->Time() - left->Time());
	}
	double wl = 1.-wr;
	
	for (i=0;i<n;i++) {
		float re,im;
		re = leftm->GetValue(i,0);
		im = leftm->GetValue(i,1);
		float a = sqrt(re * re + im * im);
		amp[i]=a*wl;
	}

	if (right)
	{
		SDIF::ConcreteMatrix<float>* rightm =
				dynamic_cast< SDIF::ConcreteMatrix<float>* >(right->GetMatrixAt(1));

		for (i=0;i<n;i++) {
			float re,im;
			re = rightm->GetValue(i,0);
			im = rightm->GetValue(i,1);
			float a = sqrt(re * re + im * im);
			amp[i]+=a*wr;
		}
	}
}

void SDIFDisplaySTF::draw(void)
{
	int i;

	mFrameIterator = mStream->Begin();
	
	int n=(*mFrameIterator)->GetMatrixAt(1)->Rows();
	
	double t= HValueOffset();
	double dt= DHValue();
	
	double dv = DVValue(); 
	
	fl_color(FL_BLACK);
	fl_rectf(x(),y(),w(),h());
	
	int cx=x();

	fl_push_clip(x(),y(),w(),h());

	double* amp = new double[n];

	fl_color(FL_WHITE);

	int xend=x()+w();
	for (cx=x();cx<xend;cx++) {
		ValuesAtTime(amp,t,mQuick);
		for (i=0;i<n;i++) {
			int k;
			k=i+int(VValueOffset());
			if (k>=0 && k<n && amp[k]) {
				if (amp[k]<0) amp[k]=0;
				if (amp[k]>1) amp[k]=1;

				fl_color(mColorMapping->Get(amp[k]));

				int cy1 = y()+h()-int(i/dv);
				int cy2 = y()+h()-int((i+1)/dv);
				fl_yxline(cx,cy1,cy2);
			}
		}
		t+=dt;
		if (mQuick) {
			t+=dt;
			cx++;
		}
	}
	fl_pop_clip();	

	delete [] amp;
}
